/* Lecture / Ecriture / Enumeration de clef et de valeur de registre **************
   Ecrit par Sebastien Bloc (sebbloc@ifrance.com)
Ex.:
	MyRegistry reg(TRUE); // autoCreate= Yes
	MyString str1;
	char str2[4096];
	MyList<MyString> lStr;
	int integer;
	LONGLONG linteger;
	BOOL r;
	r=reg="HKCU\\test";
	r=reg.SetValue("boolean");
	r=reg.SetValue("string1","hello word!");
	r=reg.SetValueExpand("path","%windir%\\myFolder");
	r=reg.SetValueMulti("listFriend","dudu\0grib\0cecile\0");
	r=reg.SetValue("integer",-12);
	r=reg.SetValueLittleEndian("integer le",(DWORD)-12);
	r=reg.SetValueBigEndian("integer be",-12);
	r=reg.SetValue("linteger",(LONGLONG) -12);
	r=reg.SetValueLittleEndian("linteger le",(LONGLONG) -12);
	r=reg.SetValue("binary",(LPBYTE)"01234567890123456789",20);

	r= (BOOL)reg.GetValue("boolean"); // res = 1;
	r=reg.GetValue("string1",str1); // "hello word !"
	r=reg.GetValue("path",str1);    // "%windir%\myFolder"
	r=reg.GetValueExpand("path",str1);  // "c:\windows\myFolder"
	r=reg.GetValue("listFriend",str1);  // "dudu grib cecile"

	r=reg.GetValue("string1",str2); // "hello word !"
	r=reg.GetValue("path",str2);    // "%windir%\myFolder"
	r=reg.GetValueExpand("path",str2);  // "c:\windows\myFolder"
	r=reg.GetValue("listFriend",str2);  // "dudu grib cecile"

	r=reg.GetValueMulti("listFriend",lStr);  // {"dudu","grib","cecile"};
	r=reg.SetValueMulti("copy listFriend",lStr);
	r=reg.GetValue("integer",&integer); // -12
	r=reg.GetValue("integer le",&integer); // -12
	r=reg.GetValue("integer be",&integer); // -12
	r=reg.GetValue("linteger",&linteger); // -12
	r=reg.GetValue("linteger le",&linteger); // -12
	LPVOID bin =reg.GetValue("binary");
	delete bin;
******************************************************************************* */

#if !defined(AFX_REGISTRE_H__E5A87928_D0D8_11D4_A7DF_00D04C390250__INCLUDED_)
#define AFX_REGISTRE_H__E5A87928_D0D8_11D4_A7DF_00D04C390250__INCLUDED_

#include "MyString.h"
#include "MyList.h"
#pragma comment(lib, "advapi32.lib")

/* ***************************************************************************************
Example:

	CRegistre reg("HKCU\Software\PlayListManager\Options");
	reg.SetValue("Param1","value1");
*************************************************************************************** */

class MyRegistry  
{
	public:
		MyRegistry (BOOL autoCreate=FALSE);					
		BOOL operator= (char *nameKey) { return Open(nameKey); }
		BOOL Open(char *nameKey);				

		void GetKeyName(MyString &result);
		BOOL Create(char *nameKey);

		void SetAutoCreate(BOOL autoCreate);
		BOOL IsAutoCreate();

		int GetValueType(char *param);
		int GetValueSize(char *param);

		// SZ | EXPAND_SZ | MULTI_SZ
		BOOL GetValue(char *param , char *result , char *defValue=NULL);		
		// SZ | EXPAND_SZ | MULTI_SZ
		BOOL GetValue(char *param, MyString &result , char *defValue=NULL);	
		// MULTI_SZ
		BOOL GetValueMulti(char *param, MyList<MyString> &result , char *defValue=NULL); 
		// EXPAND_SZ
		BOOL GetValueExpand(char *param, char *result , char *defValue=NULL); 
		// EXPAND_SZ
		BOOL GetValueExpand(char *param, MyString &result , char *defValue=NULL); 
		// DWORD | DWORD_GE | DWORD_LE
		BOOL GetValue(char *param,int *result,int defValue=0);			
		// DWORD | DWORD_GE | DWORD_LE
		BOOL GetValue(char *param,DWORD *result,DWORD defValue=0);		
		// QWORD | QWORD_LE
		BOOL GetValue(char *param,LONGLONG *result,LONGLONG defValue=0);	
		// BINARY
		LPBYTE GetValue(char *param);
		// SZ
		BOOL GetValue(char *param,float *result,float defValue=0);
		BOOL GetValue(char *param,double *result,double defValue=0);

		BOOL SetValue(char *param);									//NONE
		BOOL SetValue(char *param,int value);						//DWORD
		BOOL SetValue(char *param,DWORD value);						//DWORD
		BOOL SetValueLittleEndian(char *param,DWORD value);			//DWORD_LE
		BOOL SetValueBigEndian(char *param,DWORD value);			//DWORD_BE
		BOOL SetValue(char *param,LONGLONG value);					//QWORD
		BOOL SetValueLittleEndian(char *param,LONGLONG value);		//QWORD_LE
		BOOL SetValue(char *param,char *value);						//SZ
		BOOL SetValueExpand(char *param,char *value);				//EXPAND_SZ
		BOOL SetValueMulti(char *param,MyList<MyString> &lString);//MULTI_SZ
		BOOL SetValueMulti(char *param,char *value);				//MULTI_SZ
		BOOL SetValue(char *param,LPBYTE data,int size);			//BINARY
		BOOL SetValue(char *param,float result);					//SZ
		BOOL SetValue(char *param,double result);					//SZ

		BOOL DeleteValue(char *param);
		BOOL DeleteKey(char *nameKey=NULL);

		BOOL TestValue(char *param);
		BOOL TestKey();
		int GetLastError() { return m_lastError; }

		void BeginEnumKey() { m_numEnumKey = 0; }
		BOOL NextEnumKey(char *result);
		BOOL NextEnumKey(MyString &result);

		void BeginEnumValue() { m_numEnumValue = 0; }
		BOOL NextEnumValue (char *result);
		BOOL NextEnumValue (MyString &result);

		// MyString & operator() (char *param,char *type);// chargement /sauvegarde d'un parametre
		
	private:
		BOOL SetPrefixAndPostfix(char *nameKey);
		BOOL SetKeyName(char *nameKey);
		void HKeyToString (MyString &result);
		LPBYTE GetValue(char *param ,DWORD uTypeResult);
		BOOL SetValue(char *param,LPBYTE source,ULONG uTypeSource,ULONG sizeSource);


	private: 
		HKEY m_hKey;
		MyString m_postFixKey;
		BOOL m_lastError;

		int m_numEnumValue;
		int m_numEnumKey;

		BOOL m_isAutoCreate;
};

BOOL NextSubKey(MyString &key,MyString &subKey,MyString &suite);

#endif // !defined(AFX_REGISTRE_H__E5A87928_D0D8_11D4_A7DF_00D04C390250__INCLUDED_)
