#include "stdafx.h"
#include "MyTabCtrl.h"

#include <shlwapi.h>
#include <commctrl.h>
#include <direct.h>
#include <windowsx.h>
#include <shellapi.h>

#include "MyList.h"
#include "MyControl.h"

// ***************************************************************************************

LRESULT CALLBACK TabCtrlCallBack (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	CMyTabCtrl *tab= (CMyTabCtrl *) GetWindowLong(hDlg,GWL_USERDATA);
	if (!tab) return TRUE; // WM_DESTROY

	switch (message)
	{
		case WM_RBUTTONDOWN:
		case WM_LBUTTONDOWN:
			if (tab->iconIsButton)
			{
				int xPos = LOWORD(lParam);  // horizontal position of cursor 
				int yPos = HIWORD(lParam);  // vertical position of cursor 
 
				BOOL isOnItemIcon;
				int index = tab->HitTest(xPos,yPos,NULL,NULL,&isOnItemIcon,NULL);
				if (isOnItemIcon) tab->SetImage(index,1);
			}
			break;

		case WM_RBUTTONUP:
		case WM_LBUTTONUP:
			{
				int xPos = LOWORD(lParam);  // horizontal position of cursor 
				int yPos = HIWORD(lParam);  // vertical position of cursor 
 
				ENotifyTabCtrl eNotify;
				BOOL isOnItemIcon;
				tab->HitTest(xPos,yPos,NULL,NULL,&isOnItemIcon,NULL);
				if (isOnItemIcon) eNotify= message==WM_LBUTTONUP?TabCtrlClickOnIcon:TabCtrlClickRightOnIcon;
				else eNotify = TabCtrlClickOnItem;
				if (tab->iconIsButton&&isOnItemIcon) tab->SetImage(tab->GetCurPage(),0);

				tab->subParent.CallWindowProc(WM_COMMAND,tab->GetDlgCtrlID(),eNotify);
			}
			break;
	}

	return tab->sub.CallWindowProc(message,wParam,lParam);
}

// ***************************************************************************************

LRESULT CALLBACK TabCtrlParentCallBack (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	CMyTabCtrl *tab= (CMyTabCtrl *) GetWindowLong (hDlg,GWL_USERDATA) ;
	if (!tab) return FALSE; // WM_DESTROY

	switch (message)
	{
		case WM_NOTIFY:
			if (GetDlgItem(hDlg,wParam)==*tab)
			{
				ENotifyTabCtrl eNotify = tab->Notify(lParam);	
				if (eNotify!=TabCtrlNothing) 
					return tab->subParent.CallWindowProc(WM_COMMAND,tab->GetDlgCtrlID(),eNotify);
			}
			break;
	}
	return tab->subParent.CallWindowProc(message,wParam,lParam);
}

// ***************************************************************************************

CMyTabCtrl::CMyTabCtrl() : CMyControl()
{
}

CMyTabCtrl::CMyTabCtrl(HWND hWndTab) : CMyControl(hWndTab)
{
	Init();
}

// ***************************************************************************************

void CMyTabCtrl::Init()
{
	iconIsButton = FALSE;
	m_firstChild = TRUE;
	sub.Set(TabCtrlCallBack,(long)this);
	subParent.Set(TabCtrlParentCallBack,(long)this);
}

// ***************************************************************************************

BOOL CMyTabCtrl::operator=(HWND hWnd)
{
	CMyControl *control = (CMyControl *)this;
	*control = hWnd;
	Init();
	return TRUE;
}

/* **********************************************************************************
********************************************************************************** */

int CMyTabCtrl::Add (HWND hWndChildCurrent,char *title,BOOL show)
{
	if (!hWndChildCurrent || !title) return -1;
	
	TCITEM tcitem;
	tcitem.dwState = 0;
	tcitem.dwStateMask = 0;
	tcitem.mask = TCIF_TEXT|TCIF_IMAGE;
	tcitem.pszText= title;
	tcitem.cchTextMax = strlen(title)+1;
	tcitem.iImage = -1;
	tcitem.lParam = NULL;

	int index = TabCtrl_InsertItem(GetHWND(),GetNbPage(),&tcitem);
	if (index==-1) return -1;
	m_hWndChilds += hWndChildCurrent;

	RECT rectTab,rect,rectClient;
	if (!TabCtrl_GetItemRect(GetHWND(),index, &rectTab )) return -1;
	if (!GetClientRect (&rectClient)) return -1;
	if (!::GetClientRect (hWndChildCurrent, &rect)) return -1;

	if (!MoveWindow (
		hWndChildCurrent, 
		1, 
		rectTab.bottom+1, 
		rectClient.right-3, 
		rectClient.bottom-rectTab.bottom-3, 
		FALSE )) return -1;

	if (m_firstChild) 
	{
		ShowPage(0);
		m_firstChild = FALSE;
	}
	else
	{
		if (show) ShowPage(index);
	}
	return index;
}


/* **********************************************************************************
********************************************************************************** */

BOOL CMyTabCtrl::Sup(int index,BOOL viewNear)
{
	::DestroyWindow(m_hWndChilds[index]);
	m_hWndChilds.SuprCurrent();
	if (!TabCtrl_DeleteItem(m_hWndControl,index)) return FALSE;
	if (viewNear) ShowPage(index?index-1:0);
	return TRUE;
}


/* **********************************************************************************
********************************************************************************** */

BOOL CMyTabCtrl::Reset()
{
	m_firstChild = TRUE;

	// envoie a tout les child WM_DESTORY (pour quitter)
	for (m_hWndChilds=0;m_hWndChilds.More();m_hWndChilds++) 
		::DestroyWindow(m_hWndChilds.GetElem()); 

	m_hWndChilds.SuprAll();
	return TabCtrl_DeleteAllItems(m_hWndControl);
}


/* **********************************************************************************
********************************************************************************** */

int CMyTabCtrl::FindPage(char *title)
{
	int i;
    
	// config de la recherche
	MyString buffer;
	buffer.Resize(256);
	
	TCITEM item;
	item.mask = TCIF_TEXT;
	item.pszText = buffer;
	item.cchTextMax = 256;

	for (i=0;i<GetNbPage();i++)
	{
		if (!TabCtrl_GetItem(m_hWndControl,i,&item)) return -1;
		if (buffer == title) return i;
	}
	return -1;
}


/* **********************************************************************************
********************************************************************************** */

HWND CMyTabCtrl::operator [](char *title)
{
	int index = FindPage(title);
	if (index == -1) return 0;

	return m_hWndChilds[index];
}

/* **********************************************************************************
********************************************************************************** */

ENotifyTabCtrl CMyTabCtrl::Notify (LPARAM lParam)
{	
	LPNMHDR info = (LPNMHDR)lParam;
	ENotifyTabCtrl eNotify = TabCtrlNothing;
    switch (info->code)
	{ 
	/*case TCN_SELCHANGING:
		//case NM_CLICK:
			eNotify = TabCtrlClick;
			break;
*/
        case TCN_SELCHANGE: 
			ShowPage (GetCurPage()); 
			eNotify = TabCtrlChange; 
			break;
	}
	return eNotify;
}

/* **********************************************************************************
********************************************************************************** */

int	CMyTabCtrl::GetCurPage()
{
	return TabCtrl_GetCurSel(m_hWndControl); 
}

/* **********************************************************************************
********************************************************************************** */

int	CMyTabCtrl::GetNbPage()
{
	return TabCtrl_GetItemCount(m_hWndControl); 
}

/* **********************************************************************************
********************************************************************************** */

BOOL CMyTabCtrl::ShowPage(int index)
{	
	if (index<0 || index>=GetNbPage()) return FALSE;
	TabCtrl_SetCurSel(m_hWndControl,index);

	for (m_hWndChilds=0;m_hWndChilds.More();m_hWndChilds++)
	{
		::ShowWindow(m_hWndChilds.GetElem(),SW_HIDE);
	}	
	
	::ShowWindow(m_hWndChilds[index],SW_SHOW);

	return TRUE;
}

/* **********************************************************************************
********************************************************************************** */

void CMyTabCtrl::Highlight (int index,BOOL state)
{
	TabCtrl_HighlightItem(m_hWndControl,index,state);
}

// ************************************************************************************

BOOL CMyTabCtrl::Rename (int index, char *text)
{
	TCITEM tcitem;
	tcitem.mask = TCIF_TEXT;
	tcitem.pszText= text;
	return TabCtrl_SetItem(m_hWndControl,index,&tcitem);
}

// ************************************************************************************

BOOL CMyTabCtrl::SetImage(int indexTab,int indexImage)
{
	TabCtrl_SetImageList(GetHWND(),imageList);
	TCITEM tcitem;
	tcitem.mask = TCIF_IMAGE;
	tcitem.iImage = indexImage;
	return TabCtrl_SetItem(m_hWndControl,indexTab,&tcitem);
}

// ************************************************************************************

int CMyTabCtrl::HitTest(int x,int y,BOOL *isNoWhere,BOOL *isOnItem,BOOL *isOnItemIcon,BOOL *isOnItemLabel)
{
	TCHITTESTINFO info;
	info.pt.x = x;
	info.pt.y = y;

	int index = TabCtrl_HitTest(GetHWND(),&info);		
	if (isNoWhere) *isNoWhere = info.flags&TCHT_NOWHERE?TRUE:FALSE; //The position is not over a tab. 
	if (isOnItem) *isOnItem = info.flags&TCHT_ONITEM?TRUE:FALSE;	//The position is over a tab but not over its icon or its text. For owner-drawn tab controls, this value is specified if the position is anywhere over a tab. 
	if (isOnItemIcon) *isOnItemIcon = info.flags&TCHT_ONITEMICON?TRUE:FALSE;	//The position is over a tab's icon. 
	if (isOnItemLabel) *isOnItemLabel = info.flags&TCHT_ONITEMLABEL?TRUE:FALSE; //The position is over a tab's text.  

	return index;
}
