#include "stdafx.h"
#include "MyWindow.h"
#include "MyRect.h"

// *************************************************************************************

void DrawBitmap( HDC hDC, LONG xStart, LONG yStart, HBITMAP hBitmap) 
{   
	BITMAP bm;   
	HDC hMemDC = CreateCompatibleDC (hDC);
	POINT pt;   
	
	SelectObject (hMemDC, hBitmap);  
	GetObject (hBitmap, sizeof (BITMAP), (LPSTR) &bm);   
	pt.x = bm.bmWidth;  
	pt.y = bm.bmHeight;   
	BitBlt (hDC, xStart, yStart, pt.x, pt.y, hMemDC, 0, 0, SRCCOPY);   
	DeleteDC (hMemDC); 
}  

// *************************************************************************************

void CMyWindow::NcTitlePaint(UINT message, WPARAM wParam, LPARAM lParam)
{
	DefWindowProc (GetHWND(),message,wParam,lParam);

	CMyRect rc, rcwin; 
	CalculateTitleCaptionTextRect(&rc.r);
	
	HDC hdc = GetWindowDC(GetHWND());
	HDC dcMem = CreateCompatibleDC(hdc);

	HBITMAP bitmap = CreateCompatibleBitmap(hdc, rc.DeltaX() , rc.DeltaY());
	HBITMAP oldBitmap = (HBITMAP) SelectObject(dcMem,bitmap);

	BOOL isTool = IsToolWindow();
	CMySize sizeButton(GetSystemMetrics(isTool?SM_CXSMSIZE:SM_CXSIZE),GetSystemMetrics(isTool?SM_CYSMSIZE:SM_CYSIZE));
	CMySize sizeNormalButton(GetSystemMetrics(SM_CXSIZE),GetSystemMetrics(SM_CYSIZE));
	
	rc.r.left=rc.r.right-sizeNormalButton.dx-5;
	rc.r.top += 2;
	rc.r.bottom -= 3;
	CMyRect rcBitBlt = rc;

	for (m_titleControls=0;m_titleControls.More();m_titleControls++)
	{
		CMyEmulatedControl *control;
		control = m_titleControls.GetElemPtr();

		rc.r.left -= control->dx+3;

		switch (control->type)
		{
			case control_nothing:
				break;

			case control_check:
				if (control->state)
					 DrawBitmap(hdc,rc.r.left ,rc.r.top,control->hBitmapOn);
				else DrawBitmap(hdc,rc.r.left ,rc.r.top,control->hBitmapOff);			
				break;

			case control_button:
				if (control->pushed)
					 DrawBitmap(hdc,rc.r.left ,rc.r.top,control->hBitmapOn);
				else DrawBitmap(hdc,rc.r.left ,rc.r.top,control->hBitmapOff);
				break;
		}
	}
	
	//BOOL res = BitBlt(hdc,rc.r.left, rc.r.top, rcBitBlt.DeltaX(),rcBitBlt.DeltaY(),dcMem,rc.r.left, rc.r.top,SRCCOPY);
	SelectObject(dcMem,oldBitmap);
	DeleteObject(oldBitmap);
	ReleaseDC(GetHWND(),dcMem);
	ReleaseDC(GetHWND(),hdc);
}

// *****************************************************************************************
//cf: http://codeguru.earthweb.com/dialog/custom_dialog_caption.shtml

void CMyWindow::CalculateTitleCaptionTextRect(RECT *prc)
{
	// Get some needed system metrics.
	
	CMySize sizeFrame(GetSystemMetrics(SM_CXFRAME),GetSystemMetrics(SM_CYFRAME));
	CMySize sizeBorder(GetSystemMetrics(SM_CXBORDER),GetSystemMetrics(SM_CYBORDER));
	BOOL isTool = IsToolWindow();
	CMySize sizeButton(GetSystemMetrics(isTool?SM_CXSMSIZE:SM_CXSIZE),GetSystemMetrics(isTool?SM_CYSMSIZE:SM_CYSIZE));

	// Get the window rectangle and style information.
	
	CMyRect rcWindow;
	DWORD dwStyle = GetStyle();
	GetWindowRect(&rcWindow.r);
	
	// Set the rectangle initially based on the frame, window
	// rect, and height of a caption-bar button.
	
	if (dwStyle & WS_THICKFRAME)
	{
		prc->left = sizeFrame.dx;
		prc->top = sizeFrame.dy;
		prc->right = rcWindow.DeltaX() - sizeFrame.dx;
	}
	else
	{
		prc->left = 0;
		prc->top = 0;
		prc->right = rcWindow.DeltaX();
	}
	prc->bottom = prc->top + sizeButton.dy + 3;
	
	// If the window has a control menu, add the button width
	// and border width to the left.
	
	if (dwStyle & WS_SYSMENU)
		prc->left += sizeButton.dx + sizeBorder.dx;
	
	// Figure out the total number of border and button sizes
	// to take into account.
	
	int iBorders = 0, iButtons = 0;
	if (dwStyle & WS_MAXIMIZEBOX) iButtons++;
	if (dwStyle & WS_MINIMIZEBOX) iButtons++;
	if (iButtons > 0) iBorders++;
	if (IsTitleCloseBoxVisible()) iButtons++;
	
	prc->right -= (iBorders * sizeBorder.dx + iButtons * sizeButton.dx) - 23;
}

// *************************************************************************************

LRESULT CALLBACK CallBackTitle (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	CMyWindow *window = (CMyWindow *) GetWindowLong(hDlg,GWL_USERDATA);
	if (!window) return TRUE; // WM_DESTROY

	switch (message)
	{	
		case WM_NCLBUTTONDOWN:
			{
				POINT p;
				p.x=LOWORD(lParam);  // horizontal position of cursor 
				p.y=HIWORD(lParam);  // vertical position of cursor 

				window->m_mbClickDown = TRUE;
				CMyRect rc; 
				window->GetClientRect(&rc.r); 
				ScreenToClient(hDlg,&p);
				int nButton = window->GetTitleButtonNumber(&p);

				if (nButton != -1) 
				{
					CMyEmulatedControl *ectrl;
					ectrl=window->m_titleControls.GetElemPtr(nButton);
					ectrl->pushed = TRUE;
					switch (ectrl->type)
					{
						case control_nothing:
						case control_button:
							break;
						case control_check:
							ectrl->state=!ectrl->state;
							break;
					}
				}
				else DefWindowProc(hDlg,message,wParam,lParam);
				window->NcTitlePaint(message,NULL,NULL);
			}
			return TRUE;

		case WM_NCLBUTTONUP:
			{
				POINT p;
				p.x=LOWORD(lParam);  // horizontal position of cursor 
				p.y=HIWORD(lParam);  // vertical position of cursor 

				window->m_mbClickDown = FALSE;
				CMyRect rc; 
				window->GetClientRect(&rc.r); 
				ScreenToClient(hDlg,&p);
				int nButton = window->GetTitleButtonNumber(&p);
				if (nButton!=-1)
				{
					CMyEmulatedControl *ectrl;
					ectrl=window->m_titleControls.GetElemPtr(nButton);
					ectrl->pushed = FALSE;
					switch (ectrl->type)
					{
						case control_nothing:
							break;
						case control_button:
							window->PostMessage(WM_COMMAND,ectrl->idCtrl,NULL);
							break;

						case control_check:
							window->PostMessage(WM_COMMAND,ectrl->idCtrl,ectrl->state);
							break;
					}
				}
				window->NcTitlePaint(message,NULL,NULL);
			}
			return TRUE;
		
		case WM_NCHITTEST:
			{
				int ht = DefWindowProc(hDlg,message,wParam,lParam);
				SetWindowLong(hDlg,DWL_MSGRESULT,ht);

				if (ht == HTCAPTION)
				{
					POINT p;
					p.x=LOWORD(lParam);  // horizontal position of cursor 
					p.y=HIWORD(lParam);  // vertical position of cursor 

					ScreenToClient(hDlg,&p);
					int nButton = window->GetTitleButtonNumber(&p);				
				}
			}
			break;

		case WM_NCACTIVATE:				
		case WM_NCPAINT:
			window->NcTitlePaint(message,wParam,lParam);
			return TRUE;
	}
	
	return window->sub.CallWindowProc(message,wParam,lParam);
}


// *************************************************************************************

CMyWindow::CMyWindow()
{
}

// *************************************************************************************

CMyWindow::~CMyWindow()
{
	DeleteTitleControls();
}

// *************************************************************************************

void CMyWindow::operator=(HWND hWnd)
{
	CMyControl *control = (CMyControl *)this;
	*control = hWnd;	
	m_mbClickDown = FALSE;
	sub.Set(CallBackTitle,(long)this);

}


// *************************************************************************************

int CMyWindow::AddTitleCheck(long idCtrl,int deltaX,int deltaY,int idBitmapOn,int idBitmapOff)
{
	CMyEmulatedControl ectrl;

	ectrl.dx = deltaX;
	ectrl.dy = deltaY;
	ectrl.idCtrl = idCtrl;
	ectrl.state = FALSE;
	ectrl.type = control_check;
	ectrl.hBitmapOn = (HBITMAP) LoadImage(GethInstance(),MAKEINTRESOURCE(idBitmapOn),IMAGE_BITMAP,deltaX,deltaY,0);
	ectrl.hBitmapOff = (HBITMAP) LoadImage(GethInstance(),MAKEINTRESOURCE(idBitmapOff),IMAGE_BITMAP,deltaX,deltaY,0);
	if (!ectrl.hBitmapOn || !ectrl.hBitmapOff) return -1;
	m_titleControls+=ectrl;
	return m_titleControls.GetIndex();
}

// *************************************************************************************

int CMyWindow::AddTitleButton(long idCtrl,int deltaX,int deltaY,int idBitmapOn,int idBitmapOff)
{
	CMyEmulatedControl ectrl;

	ectrl.dx = deltaX;
	ectrl.dy = deltaY;
	ectrl.idCtrl = idCtrl;
	ectrl.state = FALSE;
	ectrl.type = control_button;
	ectrl.hBitmapOn = (HBITMAP) LoadImage(GethInstance(),MAKEINTRESOURCE(idBitmapOn),IMAGE_BITMAP,deltaX,deltaY,0);
	ectrl.hBitmapOff = (HBITMAP) LoadImage(GethInstance(),MAKEINTRESOURCE(idBitmapOff),IMAGE_BITMAP,deltaX,deltaY,0);
	if (!ectrl.hBitmapOn || !ectrl.hBitmapOff) return -1;
	m_titleControls+=ectrl;
	return m_titleControls.GetIndex();
}

// *************************************************************************************

void CMyWindow::DeleteTitleControls()
{
	for (m_titleControls=0;m_titleControls.More();m_titleControls++)
	{
		CMyEmulatedControl *ectrl;
		ectrl = m_titleControls.GetElemPtr();
		DeleteObject (ectrl->hBitmapOn);
		DeleteObject (ectrl->hBitmapOff);
	}
	m_titleControls.SuprAll();
}

// *************************************************************************************

int CMyWindow::GetTitleButtonNumber(POINT *p)
{
	CMyRect rc; 
	p->y = -p->y;
	CalculateTitleCaptionTextRect(&rc.r);

	BOOL isTool = IsToolWindow();
	CMySize sizeButton(GetSystemMetrics(isTool?SM_CXSMSIZE:SM_CXSIZE),GetSystemMetrics(isTool?SM_CYSMSIZE:SM_CYSIZE));
	CMySize sizeNormalButton(GetSystemMetrics(SM_CXSIZE),GetSystemMetrics(SM_CYSIZE));

	rc.r.left=rc.r.right-sizeNormalButton.dx-10; // PIG: -5 normalement mais sinon detection decal !!??
	rc.r.top += 2;
	rc.r.bottom -= 3;

	for (m_titleControls=0;m_titleControls.More();m_titleControls++)
	{
		CMyEmulatedControl *control;
		control = m_titleControls.GetElemPtr();

		rc.r.left -= control->dx+3;
		rc.r.right=rc.r.left+control->dx;
		if (rc.IsInside(p)) return m_titleControls.GetIndex();
	}
	return -1;
}

// *************************************************************************************

BOOL CMyWindow::IsTitleCloseBoxVisible()
{
	// In order to determine if a window has the close box, we
	// check to see if the system menu close is available.  If
	// it is, we assume the close box is present.
	
	HMENU hmenuSystem = GetSystemMenu(FALSE);
	if (hmenuSystem)
	{
		UINT uiState = GetMenuState(hmenuSystem,SC_CLOSE, MF_BYCOMMAND);
		if (!(uiState & (MF_GRAYED | MF_DISABLED)))	return TRUE;
	}
	return FALSE;
}

// *************************************************************************************

BOOL CMyWindow::SetTitleCheckState(long idCtrl,BOOL pushed)
{
	for (m_titleControls=0;m_titleControls.More();m_titleControls++)
	{
		CMyEmulatedControl *ectrl;
		ectrl = m_titleControls.GetElemPtr();
		if (ectrl->idCtrl==idCtrl)
		{
			ectrl->pushed = pushed;
			ectrl->state = pushed;
			NcTitlePaint(WM_NCLBUTTONUP,0,0);
		}
	}
	return FALSE;

}

// *************************************************************************************
// *************************************************************************************

/*
			case 0:
				if (mPushedButton[i])
					 DrawFrameControl(dcMem,&rc.r,DFC_BUTTON , DFCS_BUTTONPUSH | DFCS_CHECKED);
				else DrawFrameControl(dcMem,&rc.r,DFC_BUTTON , DFCS_BUTTONPUSH);
				break;
			case 1:
				if (mPushedButton[i])
					 DrawFrameControl(dcMem,&rc.r,DFC_BUTTON , DFCS_BUTTONCHECK | DFCS_CHECKED);
				else DrawFrameControl(dcMem,&rc.r,DFC_BUTTON , DFCS_BUTTONCHECK);
				break;
			case 2:
				if (!mPushedButton[i]) 
					 DrawFrameControl(dcMem,&rc.r,DFC_BUTTON , DFCS_BUTTONRADIO | DFCS_CHECKED);
				else DrawFrameControl(dcMem,&rc.r,DFC_BUTTON , DFCS_BUTTONRADIO);
				break;
			case 3:
				if (mPushedButton[i])
					 DrawFrameControl(dcMem,&rc.r,DFC_CAPTION ,  DFCS_CAPTIONCLOSE);
				else DrawFrameControl(dcMem,&rc.r,DFC_CAPTION , DFCS_CAPTIONHELP);
				break;
			case 4:
				if (mPushedButton[i])
					 DrawFrameControl(dcMem,&rc.r,DFC_CAPTION , DFCS_CAPTIONMIN);
				else DrawFrameControl(dcMem,&rc.r,DFC_CAPTION , DFCS_CAPTIONMAX);
				break;
			case 5:
				if (mPushedButton[i])
					 DrawFrameControl(dcMem,&rc.r,DFC_CAPTION , DFCS_CAPTIONMIN);
				else DrawFrameControl(dcMem,&rc.r,DFC_CAPTION , DFCS_CAPTIONRESTORE);
				break;
				*/
